<?php

namespace Service;

use Doctrine\ORM\EntityManager;
use Exception\InitException;
use Util\InstanceProvider;
use Util\Utilities;
use Zeedhi\DevOps\Model\Profile\Entities\ZhUser;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;

class InitService {
    /** @var \Zeedhi\DevOps\Model\Profile\Repositories\ZhUser */
    protected $zhUserRepository;

    /**
     * @var
     */
	protected $sessionManager;
    /** @var Utilities $environment */
    private $environment;

    private $profileBaseDirectory;
    /**
     * @var EntityManager
     */
    private $entityManager;

    public function __construct($sessionManager, EntityManager $entityManager, Utilities $environment ){
		$this->sessionManager = $sessionManager;
        $this->entityManager = $entityManager;
        $this->environment = $environment;
    }

	private function initSession(){
        $resourceRepository = InstanceProvider::getRepository()->getResource();
        $organizationRepository = InstanceProvider::getRepository()->getOrganization();
        /** @var Entities\Resource $currentResource */
        $currentResource = $resourceRepository->findOneBy(array('externalCode' => $this->environment->getCurrentCdoperador()));
        /** @var Entities\Organization $currentOrganization */
        $currentOrganization = $organizationRepository->find($currentResource->getOrganization());
        $this->environment->setCurrentUserId($currentResource->getId());
		$this->environment->setCurrentUserName($currentResource->getName());
		$this->environment->setCurrentUserEmail($currentResource->getEmail());
		$this->environment->setCurrentOrganizationId($currentResource->getOrganization());
		$this->environment->setCurrentOrganizationName($currentOrganization->getName());
        $this->environment->setCurrentResourceId($currentResource->getId());
    }

	public function restoreSession() {
        if($this->environment->getCurrentCdoperador()) {
            $this->initSession();
            $userReturn = array(
                "CDOPERADOR" => $this->environment->getCurrentCdoperador(),
				"USER_ID"   => $this->environment->getCurrentUserId(),
				"USER_NAME" =>$this->environment->getCurrentUserName(),
				"USER_EMAIL"   => $this->environment->getCurrentUserEmail(),
				"ORGANIZATION_NAME" => $this->environment->getCurrentOrganizationName(),
				"ORGANIZATION_ID"   => $this->environment->getCurrentOrganizationId()
			);
		} else {
            throw InitException::errorToRequestDataFromIdentity();
        }
		return $userReturn;
	}

    public function createDevOpsZhUser($userName,$userEmail,$resourceId){

        $devOpsUser = $this->zhUserRepository->findOneByExternalResId($resourceId);

        if(is_null($devOpsUser)) {
            $devOpsUser = new ZhUser();
            $devOpsUser->setName($userName);
            $devOpsUser->setEmail($userEmail);
            $devOpsUser->setExternalResId($resourceId);
            $this->environment->devOpsEnvironment->setCurrentUserId(1);
            $this->entityManager->persist($devOpsUser);
            $this->entityManager->flush();
        }

    }
    public function checkUserRecords()
    {
        $db = $this->entityManager->getConnection();
        $db->beginTransaction();
        try {
            $currentUser = $this->environment->getCurrentUser();
            $this->createDevOpsZhUser($currentUser->getName(), $currentUser->getEmail(),$currentUser->getId());
            $db->commit();
        } catch (\Exception $e) {
            $db->rollback();
            throw $e;
        }
    }

    /**
     * @param $organizationName
     * @param $externalOrgId
     * @return int
     */
    protected function createOrganization($organizationName, $externalOrgId)
    {
        $newOrganization = new Entities\Organization();
        $newOrganization->setName($organizationName);
        $newOrganization->setExternalId($externalOrgId);

        $this->entityManager->persist($newOrganization);
        $orgId = $newOrganization->getId();

        $this->entityManager->flush();
        return $orgId;
    }

    /**
     * @param $userName
     * @param $userEmail
     * @param $externalId
     * @return int
     */
    protected function createUser($userName, $userEmail, $externalId)
    {
        $newUser = new  Entities\Resource();
        $newUser->setName($userName);
        $newUser->setEmail($userEmail);
        $newUser->setExternalId($externalId);
        $newUser->setResourcesType(Entities\Resource::WORK_RESOURCE);
        $newUser->setClass(Entities\Resource::CLASS_EMPLOYEE);
        $this->entityManager->persist($newUser);
        $userId = $newUser->getId();

        $this->entityManager->flush();
        return $userId;
    }
}
